# -*- coding: utf-8 -*-
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
from pylons import session, g
from pylons.i18n import _
from waskaweb.lib.db import db
import re
import formencode
import datetime
import time
import locale

class ValidatorStateObject:

    def __setattr__(self, key, value):
        self.__dict__[key] = value

    def __getattr__(self, key):
        return self.__dict__[key]

class BaseFormValidator(formencode.Schema):
    def __init__(self):
        formencode.api.set_stdtranslation(domain="FormEncode", languages=["de"])

    allow_extra_fields = True
    filter_extra_fields = False 

class MyDateConverter(formencode.validators.FancyValidator):

    messages = {
        'wrong_format': _('Date is invalid')}

    def _to_python(self, value, state):
        value.strip()
        try:
            try:
                d = time.strptime(value, locale.nl_langinfo(locale.D_FMT))
            except:
                d = time.strptime(value, locale.nl_langinfo(locale.D_FMT).lower())
            return datetime.date(d[0], d[1], d[2])
        except:
            raise formencode.Invalid(self.message("wrong_format", state), value, state)



class MaxMinInt(formencode.validators.FancyValidator):

    """
    Convert a value to an integer.

    Example::

        >>> Int.to_python('10')
        10
        >>> Int.to_python('ten')
        Traceback (most recent call last):
            ...
        Invalid: Please enter an integer value
    """

    messages = {
        'integer': u"Bitte geben Sie einen Ganzzahlenwert ein.",
        'tooLow': u"Bitte geben Sie einen Wert größer oder gleich  %(min)i ein.",
        'tooHigh': u"Bitte geben Sie einen Wert kleiner oder gleich %(max)i ein."
        }

    min = None
    max = None

    def __initargs__(self, args):
        if self.min != None:
            self.min = int(self.min)
        if self.max != None:
            self.max = int(self.max)


    def _to_python(self, value, state):
        try:
            return int(value)
        except (ValueError, TypeError):
            raise formencode.Invalid(self.message('integer', state),
                          value, state)

    def validate_python(self, value, state):
        if self.min != None and value < self.min:
            msg = self.message("tooLow", state, min=self.min)
            raise formencode.Invalid(msg, value, state)
        if self.max != None and value > self.max:
            msg = self.message("tooHigh", state, max=self.max)
            raise formencode.Invalid(msg, value, state)

    _from_python = _to_python

class SecurePassword(formencode.validators.FancyValidator):

    min = 8
    non_letter = 2
    non_letter_literal = "zwei" 
    letter_regex = re.compile(r'[a-zA-Z]')

    messages = {
        'too_few': u'Ihr Passwort muss mindestens %(min)i '
                  u'Zeichen lang sein',
        'non_letter': u'Sie m&uuml;ssen mindestens %(non_letter)s '
                     u'Nicht-Buchstaben in Ihrem Passwort verwenden.',
        'only_ascii': u'Das Passwort darf nur aus ASCII-Zeichen bestehen.'
        }

    def _to_python(self, value, state):
        # _to_python gets run before validate_python.  Here we
        # strip whitespace off the password, because leading and
        # trailing whitespace in a password is too elite.
        return value.strip()

    def validate_python(self, value, state):
        try:
            unicode(str(value), "ascii")
        except (UnicodeDecodeError, UnicodeEncodeError):
            raise formencode.Invalid(self.message("only_ascii", state), value, state)
        if len(value) < self.min:
            raise formencode.Invalid(self.message("too_few", state, min=self.min), value, state)
        non_letters = self.letter_regex.sub('', value)
        if len(non_letters) < self.non_letter:
            raise formencode.Invalid(self.message("non_letter", state, non_letter=self.non_letter_literal), value, state)

class LoginCheck(formencode.validators.FancyValidator):

    psql_max = 63
    login_max = 0
    valid_char = re.compile(r'^[a-z_0-9]+$')

    messages = {
        'too_long': u'Die Anmeldekennung darf nicht l&auml;nger als %(max)i '
                  u'Zeichen lang sein',
        'invalid_char': u'Die Anmeldekennung darf nur Kleinbuchstaben, Zahlen oder Unterstriche enthalten',
        'login_exists': u'Es existiert bereits ein Nutzer mit der Anmeldekennung "%(login)s".'
        }

    def _to_python(self, value, state):
        # _to_python gets run before validate_python.  Here we
        # strip whitespace off the password, because leading and
        # trailing whitespace in a password is too elite.
        user = session['USER_AUTHORIZED']
        agency = user.getAgency()
        prefix_str = "ka_%s_" % agency
        prefix_length = len(prefix_str)
        self.login_max =  self.psql_max - prefix_length
        return value.strip()

    def validate_python(self, value, state):
        if len(value) > (self.login_max):
            raise formencode.Invalid(self.message("too_long", state, max=self.login_max), value, state)
        if not self.valid_char.match(value):
            raise formencode.Invalid(self.message("invalid_char", state), value, state)
        try:
            conn, cur = None, None
            loginname = "_".join(["ka", state.agency, value])
            fields = {'login': loginname}
            try:
                conn = db.getConnection()
                cur  = conn.cursor()
                cur.execute("SELECT id FROM ka_benutzer_tbl_view WHERE login = %(login)s", fields)
                result = cur.fetchone()
                if result:
                    raise formencode.Invalid(self.message("login_exists", state, login=value), value, state)
            finally:
                db.recycleConnection(conn, cur)
        except StandardError, err:
            print "Fehler beim Validieren der Anmeldekennung: %s" % err


class UserGroupExistsCheck(formencode.validators.FancyValidator):

    messages = {
        'group_exists': u'Es existiert bereits eine Benutzergruppe mit dem Namen "%(name)s"!'
        }

    def _to_python(self, value, state):
        # _to_python gets run before validate_python.  Here we
        # strip whitespace off the password, because leading and
        # trailing whitespace in a password is too elite.
        return value.strip()

    def validate_python(self, value, state):
        try:
            conn, cur = None, None
            if state:
                fields = {'name': value, 'id': state.groupid}
            else:
                fields = {'name': value, 'id': None}
            try:
                conn = db.getConnection()
                cur  = conn.cursor()
                cur.execute("SELECT name FROM ka_benutzergruppe_tbl_view WHERE id = %(id)s", fields)
                result = cur.fetchone()

                if result:
                    newname = result[0]
                else: 
                    newname = result

                if newname != value:
                    cur.execute("SELECT id FROM ka_benutzergruppe_tbl_view WHERE name = %(name)s", fields)
                    result = cur.fetchone()
                    if result:
                        raise formencode.Invalid(self.message("group_exists", state, name=value), value, state)
            finally:
                db.recycleConnection(conn, cur)
        except StandardError, err:
            print "Fehler beim Validieren des Gruppennamens: %s" % err

class NewPasswordForm(BaseFormValidator):
    passwd = SecurePassword()  
    passwd2 = formencode.validators.String() 
    chained_validators = [formencode.validators.FieldsMatch('passwd', 'passwd2')]

class NewUserGroupForm(BaseFormValidator):
    name = UserGroupExistsCheck(not_empty=True) 

class EditUserGroupForm(NewUserGroupForm):
    standin = formencode.ForEach(formencode.validators.Int(), convert_to_list=True)

class NewUserForm(BaseFormValidator):
    def __init__(self):
        formencode.api.set_stdtranslation(domain="FormEncode", languages=["de"])

    firstname = formencode.validators.String(not_empty=True) 
    lastname = formencode.validators.String(not_empty=True) 
    login = formencode.All(formencode.validators.String(not_empty=True), LoginCheck()) 
    activated = formencode.validators.StringBoolean(if_missing=False)
    newpass = formencode.validators.StringBoolean(if_missing=False)

class EditUserForm(NewUserForm):
    login = None

class CasePartSelectionForm(BaseFormValidator):
    documents = formencode.validators.Int(if_missing=0)
    appointments = formencode.validators.Int(if_missing=1)
    logbook = formencode.validators.Int(if_missing=0)
    form_section_0 = formencode.validators.Int(if_missing=0)
    form_section_1 = formencode.validators.Int(if_missing=0)
    form_section_2 = formencode.validators.Int(if_missing=0)
    form_section_3 = formencode.validators.Int(if_missing=0)
    form_section_4 = formencode.validators.Int(if_missing=0)
    form_section_5 = formencode.validators.Int(if_missing=0)
    form_section_6 = formencode.validators.Int(if_missing=0)
    form_section_7 = formencode.validators.Int(if_missing=0)
    form_section_8 = formencode.validators.Int(if_missing=0)
    form_section_9 = formencode.validators.Int(if_missing=0)
    form_section_10 = formencode.validators.Int(if_missing=0)
    form_section_11 = formencode.validators.Int(if_missing=0)
    form_section_12 = formencode.validators.Int(if_missing=0)
    form_section_13 = formencode.validators.Int(if_missing=0)
    form_section_14 = formencode.validators.Int(if_missing=0)
    form_section_15 = formencode.validators.Int(if_missing=0)
    form_section_16 = formencode.validators.Int(if_missing=0)
    form_section_17 = formencode.validators.Int(if_missing=0)
    form_section_18 = formencode.validators.Int(if_missing=0)

class NewCaseForm(BaseFormValidator):
    pass

class EditSettingsForm(BaseFormValidator):
    anon_transfer = formencode.validators.String(if_missing='off')
    max_speicherdauer = MaxMinInt(min=7, max=180, not_empty=True)

class SearchCaseForm(BaseFormValidator):
    gender   = formencode.validators.Int(if_missing='-2')
    cm_state = formencode.validators.Int(if_missing=None)

class SearchAgencyForm(SearchCaseForm):
    pass

class BundleActionForm(BaseFormValidator):
    filter_extra_fields = True 
    all_cases  = formencode.validators.Int(if_missing=0)
    action  = formencode.validators.String(not_empty=True)
    case_id = formencode.ForEach(formencode.validators.Int())

class AgencyBundleActionForm(BaseFormValidator):
    filter_extra_fields = True 
    all_cases  = formencode.validators.Int(if_missing=0)
    action  = formencode.validators.String(not_empty=True)
    agency_id = formencode.ForEach(formencode.validators.String())

class DeleteUserHelperForm(BaseFormValidator):
    filter_extra_fields = True 
    editor  = formencode.validators.Int()
    user_id = formencode.validators.Int()
    case_id = formencode.ForEach(formencode.validators.Int())


class SetEditorForm(BaseFormValidator):
    pass

class SetBundleEditorForm(BaseFormValidator):
    pass

class SetBundleStandinForm(BaseFormValidator):
    pass

class SetStandinForm(BaseFormValidator):
    filter_extra_fields = True 
    case_id = formencode.validators.Int()
    standin = formencode.ForEach(formencode.validators.Int())

class DateCheck(formencode.validators.FancyValidator):

    valid_date = re.compile(r'^[0-9]{1,2}\.[0-9]{1,2}\.[0-9]{4}$')

    messages = {
        'invalid_format': u'Bitte geben Sie das Datum im Format "TT.MM.JJJJ" an.',
        'invalid_date':   u'Bitte geben Sie ein gültiges Datum an.'
        }

    def _to_python(self, value, state):
        return value.strip()

    def validate_python(self, value, state):
        if not self.valid_date.match(value):
            raise formencode.Invalid(self.message("invalid_format", state), value, state)
        try:
            token = value.split('.')
            testdate = datetime.datetime(int(token[2]), int(token[1]), int(token[0]))
        except:
            raise formencode.Invalid(self.message("invalid_date", state), value, state)

class TimeCheck(formencode.validators.FancyValidator):

    valid_date = re.compile(r'^[0-9]{1,2}:[0-9]{1,2}$')

    messages = {
        'invalid_format': u'Bitte geben Sie die Zeit im Format "HH:MM" an.',
        'invalid_time':   u'Bitte geben Sie eine gültige Zeit an.'
        }

    def _to_python(self, value, state):
        return value.strip()

    def validate_python(self, value, state):
        if not self.valid_date.match(value):
            raise formencode.Invalid(self.message("invalid_format", state), value, state)
        try:
            token = value.split(':')
            testtime = datetime.time(int(token[0]), int(token[1]))
        except:
            raise formencode.Invalid(self.message("invalid_time", state), value, state)

class StringTooLong(formencode.validators.FancyValidator):

    messages = {
        'invalid_length': u'Bitte geben Sie einen Text mit maximal %(max)i Zeichen an.',
        }

    def __init__(self, max=80):
        self.maxlength = max

    def _to_python(self, value, state):
        return value.strip()

    def validate_python(self, value, state):
        if not value is None and len(value) > self.maxlength:
            raise formencode.Invalid(self.message("invalid_length", state, max=self.maxlength), value, state)

class CreateAppointmentForm(BaseFormValidator):
    start_date = formencode.All(formencode.validators.String(not_empty=True), DateCheck())
    start_time = formencode.All(formencode.validators.String(not_empty=True), TimeCheck())
    end_date   = DateCheck()
    end_time   = TimeCheck()
    title      = formencode.All(formencode.validators.String(not_empty=True), StringTooLong(128))

class CreateLogbookForm(BaseFormValidator):
    date         = DateCheck(not_empty=True)
    time         = TimeCheck(not_empty=True)
    duration     = formencode.validators.Int()
    short_notice = StringTooLong()

class EditLogbookForm(CreateLogbookForm):
    pass

class SetPhaseForm(BaseFormValidator):
    pass

class EvaluationFormValidator(BaseFormValidator):
    id                  = formencode.validators.String(if_missing='0')
    agencyid            = formencode.validators.String(if_missing=g.mpuls_config.get('common', 'agencyname'))
    start_date          = MyDateConverter(not_empty=True, if_missing='')
    end_date            = MyDateConverter(not_empty=True, if_missing='')
    start_date_field    = formencode.validators.String(if_missing=g.mpuls_config.get('evaloptions', 'default-start-date-field'))
    end_date_field      = formencode.validators.String(if_missing=g.mpuls_config.get('evaloptions', 'default-end-date-field'))
    typelist            = formencode.ForEach(formencode.validators.String(), convert_to_list=True)
    type_ending         = formencode.ForEach(formencode.validators.String(), convert_to_list=True)
    phase               = formencode.ForEach(formencode.validators.String(), convert_to_list=True)
    show_percent        = formencode.validators.Bool(if_missing=False)

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8
